/*  bltnsamp.c - Sample user-defined routine built from bltnstub.c

    Copyright (C) 1991, 1993, 1994, 1999, 2003, 2011, 2012 Envyr Corporation.
    All Rights Reserved.

*/

/* parameter files needed by user-defined subroutines */
#include <string.h>
#include <stdlib.h>

/* standard parameter files */
#include "status.h"
#include "calldefs.h"

#if ( OSTYPE == UNIX_TYPE )
/* any UNIX specific includes, definitions, etc.  */
#else
/* Any Windows specific includes, definitions, etc. */
#include <windows.h>
#endif


/*  location to preserve the cleanup registration function provided in the
    init_user_bltn function
*/
static status_t
    (*user_cleanup_ptr) (void (*fcn) (void)) = NULL;


/*****************************************************************************

    convert_upper

    Description:
        This routine converts the passed in string to all upper case.

    Usage:
        CALL "convert_upper" USING argument-1

        where:
            argument-1 is PIC X(n) to hold the string to be converted

    Result:
        status code

    Error Handling:
        Checks parameter count.

    Revision:
        vx1.00
            12/02/94    JMB     Based on ICOBOL version
*/

status_t convert_upper (
    argc_t
        argc,
    argv_t
        argv []
    )
{
    unsigned int
        i;


    /* arg 0 should be should be the string to be converted */
    for ( i = 0; i < argv [0].arg_len; i++ ) {
        if ( argv [0].arg_ptr [i] >= 'a' && argv [0].arg_ptr [i] <= 'z' ) {
            argv [0].arg_ptr [i] -= 32;
        }
    }
    return (SE_NO_ERR);

}   /* convert_upper */



/*****************************************************************************

    convert_lower

    Description:
        This routine converts the passed in string to all lower case.

    Usage:
        CALL "convert_lower" USING argument-1

        where:
            argument-1 is PIC X(n) to hold the string to be converted

    Result:
        status code

    Error Handling:
        Checks parameter count.

    Revision:
        vx1.00
            12/02/94    JMB     Based on ICOBOL version
*/

status_t convert_lower (
    argc_t
        argc,
    argv_t
        argv []
    )
{
    unsigned int
        i;


   /* arg 0 should be the string to be converted */
    for ( i = 0; i < argv [0].arg_len; i++ ) {
        if ( argv [0].arg_ptr [i] >= 'A' && argv [0].arg_ptr [i] <= 'Z' ) {
            argv [0].arg_ptr [i] += 32;
        }
    }
    return (SE_NO_ERR);

}  /* convert_lower */


#if ( OSTYPE == UNIX_TYPE )
/*****************************************************************************

    get_error_message

    Description:
        This routine takes an error number and returns the system
        message for that number.

    Usage:
        CALL "get_error_message" USING argument-1, argument-2

        where:
            argument-1 is PIC 9999 for the error message number
            argument-2 is PIC X(n) to store the message for argument-1

        The COBOL program should move spaces to argument-2 before
        calling get_error_message.

    Result:
        status code

    Error Handling:
        Checks parameter count.

    Revision:
        VXCOBOL 1.00
            12/02/94    JMB     Based on ICOBOL version
            12/15/03    RHJ     Updated to use strerror
*/

status_t get_error_message (
    argc_t
        argc,
    argv_t
        argv []
    )
{
    int
        error_number;
    char
        buffer [9];


    /*
       arg 0 should be the error number
       arg 1 is the return data area
    */
    if ( argv [0].arg_len > 8 ) {
        return (SE_PARM_MISMATCH);
    }
    /* else */

    /* Remember strings passed from COBOL are not null terminated,
       you must use the length.
    */
    memcpy (buffer, argv [0].arg_ptr, argv [0].arg_len);
    buffer [argv [0].arg_len] = '\0';

    /* Convert number to get error message for from character to integer */
    error_number = atoi (buffer);

    /* Now get error message */
    strncpy ((char *)argv [1].arg_ptr, strerror (error_number), argv [1].arg_len );
    return (SE_NO_ERR);

}  /* get_error_message */

#else

/*****************************************************************************

    get_error_message

    Description:
        This routine takes an error number and returns the system
        message for that number.

    Usage:
        CALL "get_error_message" USING argument-1, argument-2

        where:
            argument-1 is PIC 9999 for the error message number
            argument-2 is PIC X(n) to store the message for argument-1

        The COBOL program should move spaces to argument-2 before
        calling get_error_message.

    Result:
        status code

    Error Handling:
        Checks parameter count.

    Revision:
        06/06/11 RHJ 4.52   New
        12/31/13 DJP 4.8x   Fixed to use correct data types and correct the
                            function used to free memory
*/

status_t get_error_message (
    argc_t
        argc,
    argv_t
        argv []
    )
{
    int
        error_number;
    char
        buffer [9];
    status_t
        ret_code;
    DWORD
        ret;
    LPTSTR
        pBuffer;
#define ERRMSGBUFFERSIZE 256


    /*
       arg 0 should be the error number
       arg 1 is the return data area
    */
    if ( argv [0].arg_len > 8 ) {
        return (SE_PARM_MISMATCH);
    }
    /* else */

    /* Remember strings passed from COBOL are not null terminated,
       you must use the length.
    */
    memcpy (buffer, argv [0].arg_ptr, argv [0].arg_len);
    buffer [argv [0].arg_len] = '\0';

    /* Convert number to get error message for from character to integer */
    error_number = atoi (buffer);

    /* Now get error message */
    ret = FormatMessage(
            FORMAT_MESSAGE_ALLOCATE_BUFFER |    // The function will allocate space for pBuffer.
            FORMAT_MESSAGE_FROM_SYSTEM |        // System wide message.
            FORMAT_MESSAGE_IGNORE_INSERTS,      // No inserts.
            NULL,                               // Message is not in a module.
            error_number,                       // Message identifier.
            0,                                  // Default language.
            (LPTSTR)&pBuffer,                   // Buffer to hold the text string.
            ERRMSGBUFFERSIZE,                   // The function will allocate at least this much for pBuffer.
            NULL                                // No inserts.
        );

    if (ret != 0) {
        strncpy ((char *)argv [1].arg_ptr, pBuffer, argv [1].arg_len );
        ret_code = SE_NO_ERR;
    }
    else {
        ret_code =  SE_UNEXPECTED_ERROR;
    }
    LocalFree (pBuffer);

    return (ret_code);

}  /* get_error_message */
#endif



/* define the call table */
#if ( OSTYPE == UNIX_TYPE )
#define USER_CALL_COUNT    3
#else
#define USER_CALL_COUNT    3
#endif

/* NOTE: The name strings must be lowercase -- the runtime always converts to
         lowercase to achieve a case-insensitive lookup on program names.
*/

call_tbl_t
    user_call_tbl [USER_CALL_COUNT] = {
        /* link (NULL), name_len (0), min_args, max_args, function, name */
        { NULL, 0, 2, 2, get_error_message,      "get_error_message"    },
        { NULL, 0, 1, 1, convert_lower,          "convert_lower"        },
        { NULL, 0, 1, 1, convert_upper,          "convert_upper"        }
    };


/*****************************************************************************

    dummy_cleanup_icbltn ()

    Description:
        This routine cleans up the user builtin subsystem.

    Usage:
        called by system cleanup handler

    Parameters:
        None

    Result:
        None

    Error Handling:
        None

    Notes:
        Called at runtime process termination time.

    Revision History:
        12/05/03 DJP 3.40   New sample code
*/

static
void dummy_cleanup_icbltn (void)
{

    /* put your code here to test conditions that need cleanup and then
       do it

       For exmaple, using the scenario of the special device:

       if ( device_is_open_flag ) {
            send_control_string (device_handle, "prepare to close");
            send_control_string (device_handle, "close");
            close_device (device_handle)
            device_is_open_flag = FALSE;
       }
    */
}

/*****************************************************************************

    init_user_bltn

    Description:
        This routine initializes the user supplied builtin functions.

    Usage:
        Called by the runtime system at initialization time.

    Parameters:
        (1) Address of function which loads the calls
        (2) Address of function to register cleanup routines

    Result:
        status code

    Error Handling:
        Any errors returned will cause runtime initialization to fail.

    Revision:
        12/02/94 JMB vx1.00     New
        05/06/99 JMB vx1.51     Support for Windows
        12/05/03 DJP 3.40       Integrated UNIX/Windows support
*/

status_t init_user_bltn (
    status_t
        (*load_calls_ptr) (unsigned int call_cnt, call_tbl_t *call_tbl_ptr),
    status_t
        (*register_cleanup_ptr) (void (*fcn) (void))
    )
{
    status_t
        ret_code;


    /* protect against invalid function pointers */
    if ( load_calls_ptr == NULL || register_cleanup_ptr == NULL ) {
        return (SE_INV_OPER);
    }

    /* There are two options for cleanup:
        1) Have each function that needs a cleanup handler register its own
           handler. In this case, we need to save a copy of the registration
           procedure address globally so that they can access it.
        2) Have a single cleanup handler that handles cleanup for all
           of your routines. In this case, it can be registered immediately
           after the call table has been loaded successfully.

        An example of the need for cleanup:
            You implement a set of calls to open a special device, set up
            control information, read, write, and close it. The device needs a
            special control sequence before it is closed or it gets locked
            up. Now, suppose your COBOL program terminates unexpectedly with
            a "subscript out of bounds" error, so the close is not called.
            In this case, having a cleanup handler that assures that the
            device is properly closed will make sure the device never gets
            locked up - even if your program fails unexpctedly.

        Many routines, such as the samples in this source, don't require any
        cleanup.
    */

    /*  Method 1: save the register cleanup function pointer in case any of the
        builtin functions want to register a cleanup function via the
        register_user_cleanup_ routine found below.
    */
    user_cleanup_ptr = register_cleanup_ptr;

    /* load the call table using the provided function pointer */
    if ( failure (ret_code = (*load_calls_ptr) (USER_CALL_COUNT, &user_call_tbl [0])) ) {
        return (ret_code);
    }

    /* Method 2: register the cleanup handler now */
    if ( failure (ret_code = (*register_cleanup_ptr) (dummy_cleanup_icbltn)) ) {
        return (ret_code);
    }

    return (SUCCESS);

}   /* init_user_bltn */


/*****************************************************************************

    register_user_cleanup_

    Description:
        This routine registers a cleanup function using the registration
        function received during init_user_bltn

    Usage:
        Called by local routine wishing to register a cleanup function

    Parameters:
        Address of user defined function to be perfomred during runtime system
        termination

    Notes:
        This function is contained in the runtime system on UNIX systems
        and may be called directly.

    Result:
        status code

    Error Handling:
        Checks for null function pointer

    Revision:
        05/06/99 JMB vx1.51     New
*/

status_t register_user_cleanup_ (
    void (*cleanup_fcn) (void)
    )
{


    if ( cleanup_fcn != NULL ) {
        return ((*user_cleanup_ptr) (cleanup_fcn));
    }
    /* else */
    return (SE_INV_OPER);


}   /* register_user_cleanup_ */


/* Sample of its usage:

    ret_code = register_user_cleanup_ (dummy_cleanup_icbltn);

   or, just doing it directly with the global address:

    ret_code = (*user_cleanup_ptr) (dummy_cleanup_icbltn);

*/

