/* indxsamp - main procedure for sample program using indexed library

    Copyright (C) 1991-1999, 2001, 2002 Envyr Corporation, All Rights Reserved.

    Exported Functions:
        void
        main (argc, argv)
            standard interface to main procedure from UNIX/DOS/Windows

    Revision History:
        ic2.00
            11/12/93    LAD     Converted to IC2.00 conventions, etc...
            12/20/93    JMB     Set up a default prototype definition
        ic2.06
            03/26/95    RHJ     Added sample signal handling for SIGQUIT, SIGINT
        io2.13
            01/15/96    LAD     Changed handling of SIGQUIT to SIGTERM.
            02/14/96    LAD     Included stdlib definition of exit on MS-DOS.
            02/23/96    JMB     Excluded stdlib on AOS/VS
        05/18/98 DJP icapi2.41  Added comment re ICAPI define for DLL interface
        09/29/99 DJP            Stripped out pre-ic2.0 (1984) comments, fixed others
        11/16/99 RHJ            Updated to take multiple files
        03/28/01 RHJ icapi3.10  Cleanup
        07/02/02 RHJ    3.22    Added total_records/total_delrecords
        07/11/02 RHJ            Added handling locked records
        11/14/03 DJP 3.40       Removed IC_API_VER_0
*/

/* standard system header files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

/* ICOBOL user library header files */
#include "status.h"         /* will define ICWIN32 when on windows */
#include "system.h"
#include "indexed.h"


static
unsigned char
    record_area [IC_MAX_RECORD_SIZE_INDEXED];

static
int
    caught_signal = 0;

static
void sig_catch_signal (
    int
        sig
)
{

    caught_signal = 1;

}  /* sig_catch_signal */



/*****************************************************************************

    main - main procedure of ICOBOL programs

    Description:

    Usage:
        called by the system

    Parameters:
        argc        standard argument count parameter
        argv        standard vector of string pointers to parameters

    Result:
        none        returns exit code via exit(n) call

    Error Handling:
        The standard routines called by main are expected to report their own
        errors and return OK/FAIL.  If they return OK, main returns with an
        exit (0).  If they return FAIL, main returns with an exit(1).

    Revision History:
        io2.10
            08/29/95    JMB     Updated for for new API
        io2.13
            01/16/96    LAD     Cleanup, changed error handling slightly.
        12/11/97 JMB io2.35     Added state of the Delete-Is-Physical attribute.
        04/13/99 RHJ icapi2.50  Added state of the 4GB Big File attribute.
        07/02/02 RHJ 3.22       Added total_records/total_delrecords
*/

/* keep compilers happy and return int */
int main (
    int
        argc,       /* argument count */
    char
       *argv []     /* argument pointer vector */
    )
{
    status_t
        ret_code;
    ic_indexed_file_spec_t
        file_spec;
    ic_indexed_handle_t
        handle;
    int
        i,
        bad_files,
        loop_counter,
        key_inx;
    short unsigned int
        record_size;
    unsigned int
        total_records,
        total_delrecords,
        records_locked,
        record_count;


    printf ("Indxsamp started\n");

    /* Initialize the ICOBOL interface */
    i = ic_init_system ("IndxSamp");
    if ( i != EXIT_OK ) {
        printf ("Failure: (exit=%d) %s\n", i, "ic_init_system");
        printf ("\nIndxsamp finished.\n");
        exit (i);
    }
    /* else */
    if ( argc == 1 ) {
        ret_code = SE_INV_OPER;
        printf ("Failure: Invalid argument count.\n");
        goto end_main_sys;
    }
    /* else */

    /* Arm a couple of signal handlers */
    signal (SIGINT, sig_catch_signal);
    /* Use below under UNIX to catch Quit */
    #ifdef SIGQUIT
    signal (SIGQUIT, sig_catch_signal);
    #endif

    /* Use below under Windows to catch terminate */
    #ifdef SIGBREAK
    signal (SIGBREAK, sig_catch_signal);
    #endif
    signal (SIGTERM, sig_catch_signal);

    /* Initialize the ICOBOL Indexed file interface */
    ret_code = ic_init_indexed ();
    if ( failure (ret_code) ) {
        printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_init_indexed");
        goto end_main_sys;
    }
    /* else */
    bad_files = 0;
    loop_counter = 1;
    do {

    /* Open the requested file with the specification void */
       printf ("\nOpening %s with a void file specification...\n", argv [loop_counter]);
       file_spec.identifier = IC_FILE_SPEC_ID_INDEXED;
       ret_code = ic_open_void_indexed (argv [loop_counter],
                                           IC_OPEN_INPUT_INDEXED,
                                           &file_spec,
                                           &handle);
       if ( failure (ret_code) ) {
           printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_open_void_indexed");
           bad_files++;
           continue;
       }
       /* else */
       /* New - Added 3.22 */
       total_records = total_delrecords = 0;
       printf ("  Indexed File Specification:\n");
       printf ("    File version:          %d\n", file_spec.version);
       printf ("    Minimum record size:   %d\n", file_spec.min_record_size);
       printf ("    Maximum record size:   %d\n", file_spec.max_record_size);
       ret_code = ic_get_records_indexed (handle, &total_records);
       ret_code = ic_get_delrecords_indexed (handle, &total_delrecords);
       printf ("    Total records:         %u, Deleted records: %u\n", total_records, total_delrecords);
       printf ("    Number of keys:        %d\n", file_spec.number_of_keys);
       if ( (file_spec.attributes & IC_PURGE_ATTR_SET_INDEXED) != 0 ) {
           printf ("    Delete is %s.\n", (file_spec.attributes & IC_PURGE_ATTR_INDEXED) != 0 ? "physical" : "logical");
       }
       if ( (file_spec.attributes & IC_4GBIG_ATTR_SET_INDEXED) != 0 ) {
           printf ("    Max filesize is %s.\n", (file_spec.attributes & IC_4GBIG_ATTR_INDEXED) != 0 ? "4GB" : "2GB");
       }
       for ( key_inx = 0; key_inx < file_spec.number_of_keys; key_inx++ ) {
           printf ("        Key number %2d is at %4d:%3d\n",
                       key_inx,
                       file_spec.key [key_inx].offset,
                       file_spec.key [key_inx].size);
           if ( (file_spec.key [key_inx].opts & IC_ALSO_KEY_OPT_INDEXED) != 0 ) {
               for ( i = 0; i < file_spec.key [key_inx].count; i++ ) {
                   if ( i == 0 ) {
                       printf("            ALSO keys:      %4d:%3d\n",
                               file_spec.key [key_inx].extension.offset [i],
                               file_spec.key [key_inx].size);
                   }
                   else {
                       printf("                            %4d:%3d\n",
                               file_spec.key [key_inx].extension.offset [i],
                               file_spec.key [key_inx].size);
                   }
               }
           }
           else {
               for ( i = 0; i < file_spec.key [key_inx].extension.occurs.suffix.count; i++ ) {
                   if ( i == 0 ) {
                       printf ("            Suffixed by:    %4d:%3d\n",
                               file_spec.key [key_inx].extension.occurs.suffix.offset [0],
                               file_spec.key [key_inx].extension.occurs.suffix.size [0]);
                   }
                   else {
                       printf ("                            %4d:%3d\n",
                               file_spec.key [key_inx].extension.occurs.suffix.offset [i],
                               file_spec.key [key_inx].extension.occurs.suffix.size [i]);
                   }
               }
               if ( file_spec.key [key_inx].count != 0 ) {
                   printf ("            Occurs %2d times in %d byte intervals\n",
                               file_spec.key [key_inx].count,
                               file_spec.key [key_inx].extension.occurs.span);
               }
           }
           if ( (file_spec.key [key_inx].opts & IC_DUPL_KEY_OPT_INDEXED) != 0 ) {
               printf ("            Duplicates are permitted\n");
           }
           else {
               printf ("            Duplicates are not permitted\n");
           }
           if ( (file_spec.key [key_inx].opts & IC_UPPR_KEY_OPT_INDEXED) != 0 ) {
               printf ("            Stored in uppercase (i.e., case-insensitive)\n");
           }
           else {
               printf ("            Stored as presented (i.e., case-sensitive)\n");
           }
           if ( (file_spec.key [key_inx].opts & IC_RVRS_KEY_OPT_INDEXED) != 0 ) {
               printf ("            Stored in descending order\n");
           }
           else {
               printf ("            Stored in ascending order\n");
           }
           if ( (file_spec.key [key_inx].opts & IC_NULL_KEY_OPT_INDEXED) != 0 ) {
               printf ("            Record suppressed when all bytes are %02x (hex)\n",
                           file_spec.key [key_inx].null);
           }
       }
       ret_code = ic_close_indexed (handle);
       if ( failure (ret_code) ) {
           printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_close_indexed");
           continue;
       }
       /* else */
       if ( caught_signal != 0 ) {
           printf ("Caught signal, terminating.\n");
           goto end_main_sys;
       }
       /* else */

       /* Open the requested file with the specification */
       printf ("\nOpening %s with a (non-void) file specification...\n", argv [loop_counter]);
       ret_code = ic_open_indexed (argv [loop_counter],
                                           IC_OPEN_INPUT_INDEXED,
                                           &file_spec,
                                           &handle);
       if ( failure (ret_code) ) {
           printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_open_indexed");
           continue;
       }
       /* else */
       for ( key_inx = 0; key_inx < file_spec.number_of_keys; key_inx++ ) {
           printf ("  Start on key %d at the beginning.\n", key_inx);
           ret_code = ic_start_indexed (handle, IC_START_BEG_INDEXED, (short unsigned int) key_inx, NULL, 0);
           if ( failure (ret_code) ) {
               if ( se_get_errnum (ret_code) == SE_KEY_NOT_FND ) {
                  printf("  No records found, skipping reads.\n");
                  ic_close_indexed (handle);
                  goto next_file;
               }
               /* else */
               printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_start_indexed");
               ic_close_indexed (handle);
               goto next_file;
           }
           /* else */
           printf ("  Read next records on key %d...\n", key_inx);
           records_locked = record_count = 0;
           do {
               if ( caught_signal != 0 ) {
                   printf ("    Caught signal, terminating.\n");
                   ic_close_indexed (handle);
                   goto end_main_ind;
               }
               /* else */
               record_size = sizeof (record_area);
               ret_code = ic_read_next_indexed (handle, record_area, &record_size, IC_NULL_OPTION_INDEXED);
               if ( failure (ret_code) ) {
                   if ( se_get_errnum (ret_code) == SE_LOCKED_RECORD ) {
                      ic_read_next_indexed (handle, record_area, &record_size, IC_LOCK_IGNORE_INDEXED);
                      printf("Record locked, skipped\n");
                      ++records_locked;
                   }
                   else if ( se_get_errnum (ret_code) != SE_EOF ) {
                       printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_read_next_indexed");
                       ic_close_indexed (handle);
                       goto end_main_ind;
                   }
                   /* else */
                   continue;
               }
               /* else */
               ++record_count;
           } while ( se_get_errnum (ret_code) != SE_EOF );
           printf ("    %u records read next on key %d (locked=%u).\n", record_count, key_inx, records_locked);
       }   /* for ( key_inx < file_spec.number_of_keys ) */

       /* printf ("\n"); */
       for ( key_inx = 0; key_inx < file_spec.number_of_keys; key_inx++ ) {
           printf ("  Start on key %d at the end.\n", key_inx);
           ret_code = ic_start_indexed (handle, IC_START_END_INDEXED, (short unsigned int) key_inx, NULL, 0);
           if ( failure (ret_code) ) {
               printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_start_indexed");
               ic_close_indexed (handle);
               goto end_main_ind;
           }
           /* else */
           printf ("  Read previous records on key %d...\n", key_inx);
           records_locked = record_count = 0;
           do {
               if ( caught_signal != 0 ) {
                   printf ("    Caught signal, terminating.\n");
                   ic_close_indexed (handle);
                   goto end_main_ind;
               }
               /* else */
               record_size = sizeof (record_area);
               ret_code = ic_read_prev_indexed (handle, record_area, &record_size, IC_NULL_OPTION_INDEXED);
               if ( failure (ret_code) ) {
                   if ( se_get_errnum (ret_code) == SE_LOCKED_RECORD ) {
                      ic_read_prev_indexed (handle, record_area, &record_size, IC_LOCK_IGNORE_INDEXED);
                      printf("Record locked, skipped\n");
                      ++records_locked;
                   }
                   else if ( se_get_errnum (ret_code) != SE_EOF ) {
                       printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_read_prev_indexed");
                       ic_close_indexed (handle);
                       goto end_main_ind;
                   }
                   /* else */
                   continue;
               }
               /* else */
               ++record_count;
           } while ( se_get_errnum (ret_code) != SE_EOF );
           printf ("    %u records read previous on key %d (locked=%u).\n", record_count, key_inx, records_locked);
       }  /* for ( key_inx < file_spec.number_of_keys ) */

       ret_code = ic_close_indexed (handle);
       if ( failure (ret_code) ) {
           printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_close_indexed");
           goto next_file;
       }
       /* else */

next_file:
       ;
    } while ( ++loop_counter < argc );


    /* Terminate the ICOBOL Indexed file interface */
    ret_code = ic_term_indexed ();
    if ( failure (ret_code) ) {
        printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_term_indexed");
        goto end_main_sys;
    }
    /* else */

    /* Terminate the ICOBOL interface */
    ic_term_system ();
    if (loop_counter == 2) {
       printf ("\nIndxsamp finished.  %d argument processed, %d were bad.\n", --loop_counter, bad_files);
    }
    else {
       printf ("\nIndxsamp finished.  %d arguments processed, %d were bad.\n", --loop_counter, bad_files);
    }
    exit (EXIT_OK);

end_main_ind:
    ret_code = ic_term_indexed ();
    if ( failure (ret_code) ) {
        printf ("Failure: (errnum=%d) %s\n", se_get_errnum (ret_code), "ic_term_indexed");
    }

end_main_sys:
    ic_term_system ();
    printf ("Indxsamp finished with error(s).\n");
    exit (EXIT_JOB_ERR);
    /* keep compilers happy */
    return (EXIT_JOB_ERR);
}   /* main */

